/*
Driver.c
Original Author: Shrihari Seshadri, PSC
Version 1.00
Description: Trial & Error Sudoku Solver
*/


/* Standard libraries
Timing
String functionality
Input / Output
*/

#include <stdio.h> 
#include <string.h>
#include <time.h>
#include <stdlib.h>

#define STANDARDIZED 1

int isAll9s(char *str);
char nextChar(char c);
void increment(char **str);
int check(char *str, int puzzle[9][9]);
void makeOnes(int num, char *ans);
void fillInPuzzle(char *str, int puzzle[9][9]);
int fullSet(int rowStart, int rowFinish, int colStart, int colFinish, int
puzzle[9][9]);
void randomizePuzzle(int puzzle[9][9], int numBlanks);
void printPuzzle(int puzzle[9][9]);


int main()
{
  
  int numBlanks;
  time_t before, after;
  char *possibleAnswer;
  int puzzle[9][9] ={   {8, 7, 2, 4, 1, 6, 9, 3, 5},
                        {9, 3, 5, 2, 8, 7, 4, 1, 6},
                        {1, 4, 6, 5, 9, 3, 2, 7, 8},
                        {7, 1, 9, 6, 3, 4, 8, 5, 2},
                        {3, 5, 8, 1, 2, 9, 6, 4, 7},
                        {2, 6, 4, 7, 5, 8, 3, 9, 1},
                        {5, 8, 1, 3, 4, 2, 7, 6, 9},
                        {4, 9, 7, 8, 6, 1, 5, 2, 3},
                        {6, 2, 3, 9, 7, 5, 1, 8, 4}
  };

  srand(time(0));
  
  printf("\n\nEnter the number of blanks you want in the puzzle:\t");
  scanf("%d", &numBlanks);
  
  while(numBlanks < 0 || numBlanks > 81){
    printf("\n\nEnter the number of blanks (0 <= x <= 81) you want in the puzzle:\t");
    scanf("%d", &numBlanks);
  }
  
  printf("\n");
  
  if(numBlanks == 0){
    char *useless = "";
    if(check(useless, puzzle)) printf("The Sudoku is already correctly completed...\n\n");
    else printf("The Sudoku is already completed, but incorrectly...\n\n");
    exit(1);
  }
  
  printf("\nNumber of Blanks: %d\n", numBlanks);
  
  randomizePuzzle(puzzle, numBlanks);
  
  printPuzzle(puzzle);

  possibleAnswer = (char *) malloc(sizeof(char) * (numBlanks+1)); 
  makeOnes(numBlanks, possibleAnswer);

  before = time(NULL);

  while(1){
    if(check(possibleAnswer, puzzle)){
      printf("The solution is:\t%s\n\n", possibleAnswer);
      after = time(NULL);
      printf("The time it took to execute was: %d second(s).\n\n", (int)
difftime(after, before));
      free(possibleAnswer);
      exit(1);
    }
    
    else if(isAll9s(possibleAnswer)){
      printf("NO SOLUTION TO THE GIVEN SUDOKU.\n");
      after = time(NULL);
      printf("The time it took to execute was: %d second(s).\n\n", (int)
difftime(after, before));
      free(possibleAnswer);
      exit(1);
    }
    
    increment(&possibleAnswer);
  }
  
}

void randomizePuzzle(int puzzle[9][9], int numBlanks){

  char standardizedRandom[20];
  int row = (int) ((((float) rand()) / RAND_MAX) * 9);
  int col = (int) ((((float) rand()) / RAND_MAX) * 9);

  int i = 0, j, k;

  strcpy(standardizedRandom, "7938938224");
  
  if(STANDARDIZED && (unsigned) numBlanks <= strlen(standardizedRandom)){
    while(i < numBlanks){
      for(j = 0; j < 9; j++){
        for(k = 0; k < 9; k++){
          if(i == numBlanks) return;
          if(puzzle[j][k] == standardizedRandom[i] - '0'){
            puzzle[j][k] = -1;
            i++;
          }
        }
      }
    }
  }
  
  else{ 
    for( ; numBlanks > 0; numBlanks--){
      while(puzzle[row][col] == -1){      
        row = (int) ((((float) rand()) / RAND_MAX) * 9);
        col = (int) ((((float) rand()) / RAND_MAX) * 9);
      }
      puzzle[row][col] = -1;    
    }
  }
}

void makeOnes(int num, char *ans){

  int i;

  for(i = 0; i < num; i++)
    ans[i] = '1';

  ans[num] = '\0';

}

void printPuzzle(int puzzle[9][9]){

  int i, j;
  printf("The puzzle is:\n\n");

  for(i = 0; i < 9; i++){
    for(j = 0; j < 9; j++){
      if(puzzle[i][j] == -1) printf("_");
      else printf("%d", puzzle[i][j]);
      if(j % 3 != 2) printf(" ");
      if(j % 3 == 2 && j != 8) printf("  |  ");
    }
    
    if(i % 3 == 2 && i != 8){
      printf("\n-------------------------");
    }
    printf("\n");
  }

  printf("\n\n");
}

int isAll9s(char *str){
  int i, len = strlen(str);
  for(i = 0; i < len; i++) if(str[i] != '9') return 0;
  return 1;
}

char nextChar(char c){
  if(c == '9') return '1';
  return c + 1;
}

void increment(char **str){
  
  int len = strlen(*str), i = 1;
  
  while((*str)[len - i] == '9'){
    (*str)[len - i] = '1';
    i++;
    
  }
  
  (*str)[len - i] = nextChar((*str)[len - i]);
  
}

void fillInPuzzle(char *str, int puzzle[9][9]){
  
  int i, j, curBlank = 0;
  for(i = 0; i < 9; i++){
    for(j = 0; j < 9; j++){
      if(puzzle[i][j] == -1){
        puzzle[i][j] = str[curBlank] - '0';
        curBlank++;
      }
    }
  }  
}

int fullSet(int rowStart, int rowFinish, int colStart, int colFinish, int
puzzle[9][9]){
  
  int ans[9], i, j, ind = 0, correctIndex;
  
  for(i = rowStart; i < rowFinish + 1; i++){
    for(j = colStart; j < colFinish + 1; j++){
      ans[ind] = puzzle[i][j];
      ind++;
    }
  }
  
  for(i = 1; i < 10; i++){
    correctIndex = -1;
    for(j = 0; j < 9; j++){
      if(ans[j] == i) correctIndex = j;
    }
    if(correctIndex == -1) return 0;
  }
  
  return 1;
  
}

int check(char *str, int puzzle[9][9]){
  
  int extraCopy[9][9], i, j;

  /* extraCopy is necessary so that the actual puzzle remains unaltered */
  
  for(i = 0; i < 9; i++)
    for(j = 0; j < 9; j++)
      extraCopy[i][j] = puzzle[i][j];
  
  /*Filling in the puzzle with our possible solution*/
  
  fillInPuzzle(str, extraCopy);
  
  /*Checking rows*/
  
  for(i = 0; i < 9; i++){
    if(!fullSet(i, i, 0, 8, extraCopy)) return 0;
  }        
  
  /*Checking columns*/
  
  
  for(j = 0; j < 9; j++){
    if(!fullSet(0, 8, j, j, extraCopy)) return 0;
  }
  
  /*Checking boxes*/
  
  for(i = 0; i < 9; i++){
    int rSt = 3 * (i / 3);
    int rFi = rSt + 2;
    int cSt = 3 * (i % 3);
    int cFi = cSt + 2;
    if(!fullSet(rSt, rFi, cSt, cFi, extraCopy)) return 0;
  }
  
  /*Return 1 if the puzzle works, which it does if it has gotten here without
returning.*/  
  return 1;
}